package com.mile.v4_navigation_engine.Navigation.service


import android.app.PendingIntent;
import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.Service
import android.content.BroadcastReceiver
import android.content.Intent
import android.os.Build
import android.os.IBinder
import android.os.Looper
import android.util.Log
import androidx.core.app.NotificationCompat
import com.google.android.gms.location.*
import com.mile.v4_navigation_engine.MileNavigationEngine2
import com.mile.v4_navigation_engine.R


/**
 * Starts location updates on background and publish LocationUpdateEvent upon
 * each new location result.
 */
//class LocationUpdateService : IntentService(LocationUpdateService::class.simpleName) {
class LocationUpdateService : Service() {

    companion object {
        //region data
        private const val UPDATE_INTERVAL_IN_MILLISECONDS: Long = 1000
        const val REFRESH_DATA_INTENT = "REFRESH_DATA_INTENT"
    }

    private var mFusedLocationClient: FusedLocationProviderClient? = null
    private var locationRequest: LocationRequest? = null
    private val locationSettingsRequest: LocationSettingsRequest? = null
    var mReceiver: BroadcastReceiver? = null

    //endregion
    //onCreate
    override fun onCreate() {
        super.onCreate()
        initData()
        /*val filter = IntentFilter(REFRESH_DATA_INTENT)
        mReceiver = LocationUpdateBroadcast()
        registerReceiver(mReceiver, filter)*/
    }


    override fun onDestroy() {
        mFusedLocationClient!!.removeLocationUpdates(locationCallback)
        if(mReceiver!=null) unregisterReceiver(mReceiver);
        super.onDestroy()
    }

    //Location Callback
    private val locationCallback: LocationCallback = object : LocationCallback() {
        override fun onLocationResult(locationResult: LocationResult) {
            super.onLocationResult(locationResult)
            val currentLocation = locationResult.lastLocation
            Log.d("Locations", currentLocation.getLatitude().toString() + "," + currentLocation.getLongitude().toString());
            /*var intent = Intent(REFRESH_DATA_INTENT)
            intent.putExtra("polyline_location_service", locationResult.lastLocation)
            sendBroadcast(intent);*/
            //Share/Publish Location
        }
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        super.onStartCommand(intent, flags, startId);
        prepareForegroundNotification()
        startLocationUpdates()
        return START_STICKY
    }

    private fun startLocationUpdates() {
        mFusedLocationClient!!.requestLocationUpdates(locationRequest, locationCallback, Looper.myLooper())
    }

    private fun prepareForegroundNotification() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val serviceChannel = NotificationChannel(
                    "POLYLINE_ID",
                    "Location Service Channel",
                    NotificationManager.IMPORTANCE_DEFAULT
            )
            val manager: NotificationManager = getSystemService(NotificationManager::class.java)
            manager.createNotificationChannel(serviceChannel)
        }

        val notificationIntent = Intent(this, MileNavigationEngine2::class.java)
        val pendingIntent: PendingIntent = PendingIntent.getActivity(
                this,
                9063,
                notificationIntent,
                0)

        val notification: Notification = NotificationCompat.Builder(this, "POLYLINE_ID")
                .setContentTitle("Parcours en cours...")
                .setSmallIcon(R.mipmap.ic_launcher)
                .setContentIntent(pendingIntent)
                .build()
        startForeground(2345654, notification)
    }

    override fun onBind(intent: Intent): IBinder? {
        return null
    }

    /*override fun onHandleIntent(intent: Intent?) {
        Log.d("I DONT KNOW", "COOL")
    }*/

    private fun initData() {
        locationRequest = LocationRequest.create()
        locationRequest!!.setInterval(UPDATE_INTERVAL_IN_MILLISECONDS)
        locationRequest!!.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY)
        mFusedLocationClient = LocationServices.getFusedLocationProviderClient(applicationContext)
    }
}